@@ -2,18 +2,22 @@
 text_document_did_open <- function(self, params) {
     textDocument <- params$textDocument
     uri <- textDocument$uri
-    self$documents[[uri]] <- readLines(path_from_uri(uri))
-    self$sync_input_queue$put(uri, NULL)
+    doc <- readLines(path_from_uri(uri), warn = FALSE)
+    self$documents[[uri]] <- doc
+    self$sync_input_dict$set(uri, doc)
 }
 
 # Notification
 text_document_did_change <- function(self, params) {
     textDocument <- params$textDocument
     contentChanges <- params$contentChanges
-    text <- contentChanges$text
+    text <- contentChanges[[1]]$text
     uri <- textDocument$uri
-    self$documents[[uri]] <- stringr::str_split(text, "\n")[[1]]
-    self$sync_input_queue$put(uri, self$documents[[uri]])
+    doc <- stringr::str_split(text, "\n")[[1]]
+    # remove last empty line
+    if (nchar(doc[[length(doc)]]) == 0) doc <- doc[-length(doc)]
+    self$documents[[uri]] <- doc
+    self$sync_input_dict$set(uri, doc)
 }
 
 # Notification
@@ -25,8 +29,8 @@ text_document_will_save <- function(self, params) {
 text_document_did_save <- function(self, params) {
     textDocument <- params$textDocument
     uri <- textDocument$uri
-    self$documents[[uri]] <- readLines(path_from_uri(uri))
-    self$sync_input_queue$put(uri, NULL)
+    self$documents[[uri]] <- readLines(path_from_uri(uri), warn = FALSE)
+    self$sync_input_dict$set(uri, NULL)
 }
 
 # Notification
