@@ -1,7 +1,17 @@
-hover_reply <- function(id, workspace, document, position) {
+hover_reply <- function(id, uri, workspace, document, position) {
     line <- position$line
     character <- position$character
 
+    if (!check_scope(uri, document, line)) {
+        Response$new(
+            id,
+            result = list(
+                contents = NULL
+            )
+        )
+        return(invisible(NULL))
+    }
+
     hover <- detect_hover(document, line, character)
 
     logger$info("hover: ", hover)
