@@ -1,7 +1,18 @@
-signature_reply <- function(id, workspace, document, position) {
+signature_reply <- function(id, uri, workspace, document, position) {
     line <- position$line
     character <- position$character
 
+    if (!check_scope(uri, document, line)) {
+        Response$new(
+            id,
+            result = list(
+                signatures = NULL,
+                activeSignature = 0
+            )
+        )
+        return(invisible(NULL))
+    }
+
     closure <- detect_closure(document, line, character)
 
     SignatureInformation <- list()
